import os

from prometheus_client import CollectorRegistry, CONTENT_TYPE_LATEST, REGISTRY, generate_latest
from prometheus_client.multiprocess import MultiProcessCollector
from starlette.requests import Request
from starlette.responses import Response


def metrics(request: Request) -> Response:
    if "prometheus_multiproc_dir" in os.environ:
        registry = CollectorRegistry()
        MultiProcessCollector(registry)
    else:
        registry = REGISTRY

    return Response(generate_latest(registry), media_type=CONTENT_TYPE_LATEST)



import time

from prometheus_client import Counter, Gauge, Histogram
from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
from starlette.requests import Request
from starlette.types import ASGIInstance

REQUESTS = Counter("starlette_requests_total", "Total count of requests by mathod and path.", ["method", "path"])
RESPONSES = Counter(
    "starlette_responses_total",
    "Total count of responses by method, path and status codes.",
    ["method", "path", "status_code"],
)
REQUESTS_PROCESSING_TIME = Histogram(
    "starlette_requests_processing_time_seconds",
    "Histogram of requests processing time by path (in seconds)",
    ["method", "path"],
)
EXCEPTIONS = Counter(
    "starlette_exceptions_total",
    "Histogram of exceptions raised by path and exception type",
    ["method", "path", "exception_type"],
)
REQUESTS_IN_PROGRESS = Gauge("starlette_requests_in_progress", "Gauge of requests currently being processed")


class PrometheusMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint) -> ASGIInstance:
        method = request.method
        path = request.url.path

        REQUESTS_IN_PROGRESS.inc()
        REQUESTS.labels(method=method, path=path).inc()
        try:
            before_time = time.time()
            response = await call_next(request)
            after_time = time.time()
        except Exception as e:
            EXCEPTIONS.labels(method=method, path=path, exception_type=type(e).__name__).inc()
            raise e from None
        else:
            REQUESTS_PROCESSING_TIME.labels(method=method, path=path).observe(after_time - before_time)
            RESPONSES.labels(method=method, path=path, status_code=response.status_code).inc()
        finally:
            REQUESTS_IN_PROGRESS.dec()

        return response



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['starlette_prometheus']

package_data = \
{'': ['*']}

install_requires = \
['prometheus_client>=0.5.0,<0.6.0', 'starlette>=0.9.0']

setup_kwargs = {
    'name': 'starlette-prometheus',
    'version': '0.1.0',
    'description': 'Prometheus integration for Starlette',
    'long_description': '# Starlette Prometheus\n[![Build Status](https://travis-ci.org/PeRDy/starlette-prometheus.svg?branch=master)](https://travis-ci.org/PeRDy/starlette-prometheus)\n[![codecov](https://codecov.io/gh/PeRDy/starlette-prometheus/branch/master/graph/badge.svg)](https://codecov.io/gh/PeRDy/starlette-prometheus)\n[![PyPI version](https://badge.fury.io/py/starlette-prometheus.svg)](https://badge.fury.io/py/starlette-prometheus)\n\n* **Version:** 0.1.0\n* **Status:** Production/Stable\n* **Author:** José Antonio Perdiguero López\n\n## Introduction\n\nPrometheus integration for Starlette.\n\n## Requirements\n\n* Python 3.6+\n* Starlette 0.9+\n\n## Installation\n\n```console\n$ pip install starlette-prometheus\n```\n\n## Usage\n\nA complete example that exposes prometheus metrics endpoint under `/metrics/` path.\n\n```python\nfrom starlette.applications import Starlette\nfrom starlette_prometheus import metrics, PrometheusMiddleware\n\napp = Starlette()\n\napp.add_middleware(PrometheusMiddleware)\napp.add_route("/metrics/", metrics)\n```\n\n## Contributing\n\nThis project is absolutely open to contributions so if you have a nice idea, create an issue to let the community \ndiscuss it.',
    'author': 'José Antonio Perdiguero López',
    'author_email': 'perdy@perdy.io',
    'url': 'https://github.com/PeRDy/starlette-prometheus',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
